###########################################################################
### Win32 XmlRpcFilteringPipe clipboard agent
###
###########################################################################

use strict;
use re;
use Win32;
use Win32::API;
use Win32::GUI;   
use Win32::Clipboard;
use SOAP::Lite;
use SOAP::Transport::HTTP;
use LWP::UserAgent;
use XML::Parser::Lite;
use XMLRPC::Lite;

our $icon_fn = "xrfp_clipboard.ico";

our ($hwnd, $icon, $hwnd_class, $window, $menu_bar);
our ($img, $systray_icon, $systray_menu, $font, $logbox);
our ($alert_window, $screen_width, $screen_height);

# used to circumvent a bug where the
# popup menu functions get called twice.
our $already_called = 0;

gui_init();
my $rc = Win32::GUI::Dialog(0);

###########################################################################
### get_filters
###########################################################################
sub get_filters {
    my @filters = 
        (
            ["http://www.decafbad.com/twiki/bin/twiki_xmlrpc.cgi", 
                "wiki.filterData"],
            ["http://www.decafbad.com/xrfp_html2text.cgi",
                "html2text.filterData"]
        );

    return \@filters;
}

###########################################################################
### filter_clipboard
###########################################################################
sub filter_clipboard {
    my ($filters) = @_;

    my $cb = Win32::Clipboard();
    my $textIn = $cb->Get();
    my $textOut = $textIn;

    foreach my $filter (@$filters) {
        my ($url, $method) = @$filter;
        my $proxy  = XMLRPC::Lite->proxy($url);

        $textOut = $proxy->call($method, $textOut)->result();
    }

    $cb->Set($textOut);
}

###########################################################################
### _FilterClipboard_Click
###########################################################################
sub _FilterClipboard_Click { 
    ### Debounce the systray menu
    unless ( $already_called ) {
        $already_called = 1;

        my $filters = get_filters();
        filter_clipboard($filters);
        
        my $msg = "Clipboard processed with the following filters:\n\n";
        $msg .= join( "\n", map { '     ["'.$_->[0].'"].'.$_->[1] } @$filters);
        $msg .= "\n\n";
        $msg .= "Share and enjoy.";        
        alert_box($msg);
   } else { 
        $already_called = 0; 
   }
}

###########################################################################
### gui_init
###########################################################################
sub gui_init {

   use Win32::GUI;

   $hwnd = GUI::GetPerlWindow();
   GUI::Hide($hwnd);

   # create the icon handler.
   $icon = new Win32::GUI::Icon($icon_fn);

   # create a window class for our window.
   $hwnd_class = new Win32::GUI::Class
        (-name => "XRFP Agent", -icon => $icon);

   # creates the main window. notice that we use a generic
   # "Name" parameter, which is used for events, which must
   # have the format Name_EventName.
   $window = new Win32::GUI::Window( -name    => '_Window', 
                                     -text    => "XRFP Agent",
                                     -class   => $hwnd_class,
                                     -icon    => $icon,
                                     -maximizebox => 0
                                   );

   # create the systray icon.
   $systray_icon = $window->AddNotifyIcon( -name => "_Systray",
                                           -id   => 1,
                                           -icon => $icon,
                                           -tip  => "XRFP Agent"
                                         );

   # create the popup menu.
   $systray_menu = new Win32::GUI::Menu
        ( "SystrayMenu Functions" => "SystrayMenu",
             "> Apply Filters to Clipboard" => "_FilterClipboard",
             "> Exit" => "_SystrayExit" );

   return 1;
}

sub alert_box {
    my ($msg) = @_;
    
    Win32::MsgBox($msg, Win32::MB_ICONINFORMATION, "XRFP Agent");
}

sub _Remove_Systray_Icon { 
    Win32::GUI::NotifyIcon::Delete
        ($systray_icon->{-parent}, -id=>$systray_icon->{-id} ); 
}

sub _Systray_Click { 
    _Systray_RightClick()
}

sub _Systray_RightClick { 
   # get the x and y coords of the mouse to display the menu at.
   my($x, $y) = Win32::GUI::GetCursorPos();

   # make the popup menu visible at the cursor
   $window->TrackPopupMenu($systray_menu->{SystrayMenu}, $x, $y-50);
}

sub _SystrayExit_Click { 
    &exit_program; 
}

sub exit_program {
   # remove the icon.
   &_Remove_Systray_Icon;

   # ciao.
   exit;
}
